#define	MODULE_NAME	"MAIN"

#include "TOOLS.h"


////////////////////////////////////////////////////////////////////////////////
//
//	Display command usage
//
void	usage (char * command)
{
	printf ("Usage : %s [<option>] <instrument> <yyyy-mm-dd> <antenna (hexadecimal>\n", basename (command));
	printf ("\n");
	printf ("Download BepiColombo MMO data for the MEA instrument\n");
	printf ("\n");
	printf ("With <instrument> from MEA1|MEA2\n");
}


////////////////////////////////////////////////////////////////////////////////
//
//	Main program
//
int	main (int argc, char ** argv)
{
	char *		fname = FNAME ("main");
	t_err		error = OK;
	t_date		date ;
	char *		instrument;
	int		antenna;
	char		key [15];

	if (argc != 4) {

		usage (argv [0]);
		goto EXIT;
	}

	instrument = argv [1];

	if (Error (error = str_to_date (argv [2], & date))) goto EXIT;

	if (sscanf (argv [3], "%x", & antenna) != 1) {

		Error_log (fname, "Uncorrect value for antenna : %s", argv [3]);
		error = ERROR;
		goto EXIT;
	}

	if (Error (Check_BEPI_root())) goto EXIT;

	if (Error (error = Create_directory (Get_L0_directory (instrument, date)))) {

		Error_log (fname, "Erreur creation repertoire %s", Get_L0_directory (instrument, date));
		goto EXIT;
	
	}

	sprintf (key, "TLM_%03X", antenna);

	if (Error (error = Set_logfile (Get_L0_filename (instrument, date, key, "log")))) goto EXIT;

	if (Error (error = SDT_open_mmo (instrument, date, antenna))) {

		Error_log (fname, "Error connection : %d", error);
		goto EXIT;
	}

	error = SDT_read_packets (instrument, date, antenna);

EXIT:	return error;
}
