/*****************************************************************************************
 *  ユーザ提供SDTP関数サンプルプログラム (データ蓄積、SIRIUS共通)
 *  
 *  呼び出し形式  
 *     int READ_sdtp_mmo() ;
 *  
 *		return
 *      0 : 正常終了
 *		1 : タイムアウト
 *		2 : 通信解放依頼受信 (closeをする)
 *		10: データが無し(該当データが存在しなかった)
 *		20: データ終了
 *		0<: 異常終了　SDTP READのエラー情報
 *  
 *  DATE    2012.06.18  作成
 *
 ****************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#ifdef WIN32
#include    <windows.h>
#include    <winsock2.h>
#else
#include    <netinet/in.h>
#endif


#include    "SDT_all.h"
#include    "SDT_tlm.h"
#include    "MMO_sdtp.h"

char * hex_str (unsigned char * str, int size)
{	
	static	char buffer [512];
	int	i;

	strcpy (buffer, "");
	for (i = 0; i < size; i++)
		sprintf (buffer, "%s%02X ", buffer, str [i]);
	return buffer;
}

#define __DEBUG

int READ_sdtp_mmo()
{
    int             ret,
                    i,
                    leng,
                    posi;
    int             ia;		/* apid set position */
    int             iflg;	/* 0=data end 1:first data * *
				 * 2>=:continuation data */
    int             iseq_flg;	/* packet sequence flag */
    int             iplen;	/* ccsds length */
    unsigned int    iapid,
                    iseqcnt;	/* apid , sequence count */
    unsigned char   cpnbuf[128];
    unsigned char   wk[16];
    int             pdu_type,
                    recv_len;
    unsigned char   recv_data[65536];	/* recive data */
    union Wd {
	int             iwd;
	unsigned char   cwd[4];
    } wd;
    int             j;

    int             APID_numget(unsigned int);

 /*******************************************************/
    /*
     * 初期設定　　　　　　　 
     */
 /*******************************************************/
    ipkt_cnt = 0;
    for (i = 0; i < 64; i++) {
	ipkt_len[i] = 0;
    }
    memset(P_tlm, 0x00, sizeof(P_tlm));

 /*******************************************************/
    /*
     * ＳＤＴＰデータ受信処理 
     */
 /*******************************************************/
    ret = SDT_tlm_recv(&pdu_type, recv_data, &recv_len);

#ifdef DBG
    printf("ICI2 SDT_tlm_recv=%d pdu_type=%d recv_len=%d\n", ret, pdu_type, recv_len);
#endif

    if (ret < 0) {
	/*
	 * エラー発生 
	 *
	 * printf("SDT_tlm_recv() Error [%d]\n", ret ); 
	 */
	return (ret);
    }
    if (ret == 1) {
	/*
	 * Time Out 
	 */
	return (ret);
    } else if (ret == 2) {
	/*
	 * 通信解放依頼受信 
	 *
	 * printf("Free Request PDU Receive\n"); 
	 */
	return (ret);
    }
    /*
     * PDUデータ受信 
     */

    if (pdu_type == 0) {
	leng = 0;
	posi = 0;
#ifdef	DEBUG
	printf("recv_len=%d\n", recv_len);
#endif

	/*
	 * ブロック数分処理を行う 
	 */
	for (i = 0; i < blk_no; i++) {

int	ccsds_len, seqcount, ccsds_size, adu_size;

	    /*
	     * データ長取り出し 
	     */
	    memcpy(&wd.cwd, &recv_data[posi + 8], 4);
	    leng = ntohl(wd.iwd);

#ifdef DEBUG
	    printf("[%d]i=%d recv_data=%x leng=%d\n", __LINE__, i, recv_data[posi], leng);
#endif

	    /*
	     * 先頭データ 
	     */
	    if (recv_data[posi + 0] == 0x01) {			// Top data
		memcpy(wk, &recv_data[posi + 12 + 1], 5);
		sat_no = (int) recv_data[posi + 12 + 6];
		sprintf(path_no, "%02x%02x%02x%02x%02x\n", wk[0], wk[1],
			wk[2], wk[3], wk[4]);
		/*
		 * 最終データ
		 */
	    } else if (recv_data[posi + 0] == 0x04) {		// Final data
		return (20);
		/*
		 * 中間データ 
		 */
	    } else if (recv_data[posi + 0] == 0x02) {		// Intermediate data
#ifdef DBG
		printf("[%d]i=%d  \n", __LINE__, i);
#endif
		/*
		 * パケット 
		 *	
		 * data length set 
		 */
		iplen = leng;
		/*
		 * data set 
		 */
		memcpy(&read_pkt, &recv_data[posi + 12 + 0], iplen);
		iseq_flg = read_pkt.p_fg_cnt & 0xc0;
		iseq_flg = iseq_flg / 64;

		iapid = (int) (read_pkt.pkt_hed[0] & 0x07) * 256;
		iapid = iapid + read_pkt.pkt_hed[1];

		seqcount = (read_pkt.pkt_hed[2] & 0x3f) * 256 + read_pkt.pkt_hed[3];

ccsds_len = read_pkt.pkt_hed[4] * 256 + read_pkt.pkt_hed[5];
ccsds_size = (iseq_flg == 3) ? 8: 12;
adu_size = (iseq_flg == 3) 
	? 0
	: (read_pkt.adu_lng[0] * 256 + read_pkt.adu_lng[1]) * 256 + read_pkt.adu_lng[2];

		iseqcnt = (int) (read_pkt.psq_cnt[0] * 256) + (int) read_pkt.psq_cnt[1];
#ifdef	DEBUG
	printf ("Blk %2d : %05d : %s \n", i, posi, hex_str (recv_data + posi, 12));
	printf ("CCSDS H1  : %s...\n", hex_str (recv_data + posi + 12 + 64, 6));
	printf ("CCSDS H2  : %s...\n", hex_str (recv_data + posi + 12 + 64 + 6, ccsds_size));
	printf ("APID : %4X FLAG = %1d SEQ = %4d CCSDS len = %4d\n", iapid, iseq_flg, seqcount, ccsds_len);
	printf ("iplen = %d ccsds_len = %d adu_size = %d\n", iplen, ccsds_len, adu_size);
	printf ("trailer = %02x\n", recv_data[posi + 12 + iplen -1]);
#endif
#ifdef DBG
		printf ("[%d]i=%d len=%d seq_flg=%d APID=%04x plen=%02x%02x data[%02x%02x%02x%02x%02x%02x] \n",
		     __LINE__, i, iplen, iseq_flg, iapid,
		     read_pkt.pkt_hed[4], read_pkt.pkt_hed[5],
		     recv_data[posi + 12 + 64], recv_data[posi + 12 + 65],
		     recv_data[posi + 12 + 66], recv_data[posi + 12 + 67],
		     recv_data[posi + 12 + 68], recv_data[posi + 12 + 69]
		    );
#endif

		/*
		 * unsegmented User Data 
		 */
		if (iseq_flg == 3) {
		    memcpy(&P_tlm[ipkt_cnt], &read_pkt, iplen);
		    ipkt_len[ipkt_cnt] = iplen;
		    ipkt_cnt++;

		    /*
		     * first segment User Data 
		     */
		} else if (iseq_flg == 1) {
		    ia = APID_numget(iapid);
#ifdef DBG
		    printf("[%d] seq_flg=%d ia=%d apid=%x \n", __LINE__, iseq_flg, ia, iapid);
#endif
     		    /** データ途中です　**/
		    if (iseg_flg[ia] != 0) {
			printf(" no last segment data apid=%04x\n", iapid);
		    }

		    memcpy(&save_pkt[ia], &read_pkt, iplen);
		    iseg_flg[ia] = 1;	/* flag=first set */
		    ip_lenSav[ia] = iplen;	/* packet total * Length
						 * Set */
		    iseq_cnt[ia] = iseqcnt;

		    /*
		     * continueation segment User Data 
		     */
		} else if (iseq_flg == 0) {
		    ia = APID_numget(iapid);
		    /*
		     * nothin first data ERR 
		     */
		    if (iseg_flg[ia] == 0) {
			printf(" no first segment data apid=%04x\n",
			       iapid);
			ip_lenSav[ia] = 0;
			iseg_flg[ia] = 0;
			iseq_cnt[ia] = 0;
			continue;
			/*
			 * data set 82->64+6+12
			 * 付加情報+パケットヘッダ+2ndヘッダ 
			 */
		    } else {
			memcpy(&save_pkt[ia].t_frame[ip_lenSav[ia] - 82],
			       read_pkt.t_frame, (iplen - 82));
			iseg_flg[ia]++;
			ip_lenSav[ia] = ip_lenSav[ia] + (iplen - 82);

			/*
			 * sequence count check 
			 */
			if (iseq_chk == 1) {
#ifdef DBG
			    printf("[%d] iseqcnt=%d iseq_cntsave=%d \n", __LINE__, iseqcnt, iseq_cnt[ia]);
#endif
			    if (iseqcnt != iseq_cnt[ia] + 1) {
				printf
				    (" Sequence cout Error Bef=%d Aft=%d\n",
				     iseq_cnt[ia], read_pkt.psq_cnt);
			    }
			}
			iseq_cnt[ia] = iseqcnt;
		    }

		    /*
		     * last segment User Data 
		     */
		} else if (iseq_flg == 2) {
		    ia = APID_numget(iapid);
		    /*
		     * nothin first data ERR 
		     */
		    if (iseg_flg[ia] == 0) {
			printf(" no first segment data apid=%04x\n",
			       iapid);
			ip_lenSav[ia] = 0;
			iseg_flg[ia] = 0;
			iseq_cnt[ia] = 0;
			continue;
			/*
			 * data set 
			 */
		    } else {
			memcpy(&save_pkt[ia].t_frame[ip_lenSav[ia] - 82],
			       read_pkt.t_frame, (iplen - 82));
			/*
			 * 最終データのADU　Length を設定 
			 */
			memcpy(save_pkt[ia].adu_lng, read_pkt.adu_lng, 3);

			ip_lenSav[ia] = ip_lenSav[ia] + (iplen - 82);
			iseg_flg[ia] = 0;

			/*
			 * sequence count check 
			 */
			if (iseq_chk == 1) {
#ifdef DBG
			    printf("[%d] iseqcnt=%d iseq_cntsave=%d \n", __LINE__, iseqcnt, iseq_cnt[ia]);
#endif
			    if (iseqcnt != iseq_cnt[ia] + 1) {
				printf
				    (" Sequence cout Error Bef=%d Aft=%d\n",
				     iseq_cnt[ia], read_pkt.psq_cnt);
			    }
			}
			iseq_cnt[ia] = iseqcnt;

			/*
			 * ユーザ領域に設定 
			 */
			memcpy(&P_tlm[ipkt_cnt], &save_pkt[ia],
			       ip_lenSav[ia]);
			ipkt_len[ipkt_cnt] = ip_lenSav[ia];
			ipkt_cnt++;
			ip_lenSav[ia] = 0;
		    }
		}

	    }
	    /*
	     * 次のブロック先頭ポインタ 
	     */
	    posi = posi + 12 + leng;
#ifdef DBG
	    printf("[%d]posi=%d len=%d \n", __LINE__, posi, recv_len);
#endif
	    /*
	     * 受信長分終了ならば次の受信へ 
	     */
	    if (posi >= recv_len)
		break;
	}
    }

    if (ipkt_cnt > 0) {
	return (0);
    } else {
	return (10);
    }

}

/**************************************
 *
 *    APID CHECK
 *
****************************************/
int
APID_numget(unsigned int iap)
{
    int             i;

    /*
     * apid check 
     */
    for (i = 0; i < PKTCNT; i++) {
#ifdef DBG
	printf("%s[%d]save_pno=%x apid=%x \n", __FILE__, __LINE__,
	       save_pno[i], iap);
#endif
	if (save_pno[i] == -1) {
	    save_pno[i] = iap;
	    return (i);
	}
	if (save_pno[i] == iap) {
	    return (i);
	}
    }
    return (-1);
}
