////////////////////////////////////////////////////////////////////////////////
//
//	Module	SDT
//
//	Parametrisation des acces au reseau SDTP pour MMO
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"SDT"

#include <assert.h>
#include "SDT_all.h"
#include "SDT_tlm.h"

#include "TOOLS.h"
#include "DEFS.h"

#include "SDT.h"
#include "MMO_sdtp.h"
#include "rice.h"

#define	DELAY_MS	1000		// Delay after time out before retry (ms)

#define	MAX_TIME_OUT	50		// Max number of timeout before leaving

#define	BUFSIZE		8192

typedef	struct {			// Descriptor of (APID,VCID) tuples
	int	apid;		
	int	vcid;
}	t_descr;

static	t_descr	MDP1_DESCR [] = {

	0x619,		VCID_REPRO,	// HKD DPU1
	0x619,		VCID_REAL,	// HKD DPU1
	EOF, EOF
};

static	t_descr	MIA_DESCR [] = {

	0x538,		VCID_REAL,	// Mission MIA
	0x538,		VCID_REPRO,
};

static	t_descr	MEA1_DESCR [] = {	// Is it necessary to set VCID_REPRO and VCID_REAL for each APID ???
	
 	APID_MDP1_DUMP,	VCID_REPRO,	
	APID_MEA1_HSK,	VCID_REPRO,
	APID_MEA1_SCI,	VCID_REPRO,

	APID_MDP1_DUMP,	VCID_REAL,
	APID_MEA1_HSK,	VCID_REAL,
	APID_MEA1_SCI,	VCID_REAL,

	0x428,		VCID_REPRO,	// REPORT ??
	0x428,		VCID_REAL,
*/
	EOF,		EOF
};

static	t_descr	MEA2_DESCR [] = {

	APID_MDP1_DUMP,	VCID_REPRO,	
	APID_MEA2_HSK,	VCID_REPRO,
	APID_MEA2_SCI,	VCID_REPRO,

	APID_MDP1_DUMP,	VCID_REAL,
	APID_MEA2_HSK,	VCID_REAL,
	APID_MEA2_SCI,	VCID_REAL,

	0x430,		VCID_REPRO,	// REPORT ???
	0x430,		VCID_REAL,
	EOF,		EOF
};

static 	union {

	int32_t	iwd;
	int8_t	cwd[4];
}	wd;

typedef	char 	STR_TIME [32];

static	uint8_t	uncompress_buffer [BUFSIZE];


////////////////////////////////////////////////////////////////////////////////
//	
//	Clear or set global variables and data structures
//
void	SDT_clear ()
{
	int	i;

	iseq_chk = 1;

	blk_no = PKTCNT;

	for (i = 0; i < PKTCNT; i++) {

		save_pno [i] = -1;
		iseg_flg [i] = 0;
		iseq_cnt [i] = 0;
		ip_lenSav [i] = 0;
	}

	memset (save_pkt, 0, sizeof(save_pkt));
}


////////////////////////////////////////////////////////////////////////////////
//
//	Check opened connection
//
t_err	SDT_Confirm (void)
{
	char * 		fname = FNAME ("SDT_Confirm");
	t_err		error = OK;
	int		status;
	int		pdu_type, recv_len;
	unsigned char	recv_data [BUFSIZ];
	unsigned char	wk [16];
	int	i;	

	status = SDT_tlm_recv (& pdu_type, recv_data, & recv_len);

	if (status < 0 || status == 2) {

		Error_log (fname, "tlm_recv error %d", status);
		SDT_tlm_close();
		error = ERROR;
		goto EXIT;
	}

	Message_log (1, fname, "recv_len = %d", recv_len);

#ifdef	DEBUG
	for (i = 0; i < recv_len; i++) printf ("%02X ", recv_data [i]);
	printf ("\n");
#endif

	if (recv_data [0] == 0x01) {

		memcpy (wk, & recv_data [12+1], 5);
		sprintf (path_no, "%02X%02X%02X%02X%02X", wk[0], wk[1], wk[2], wk [3], wk[4]);
		sat_no = (int) recv_data [12+6];

		Message_log (1, fname, "Sat = %d, Path_no = %s", sat_no, path_no);
	}

EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Open SDT connection 
//
int	SDT_open_mmo (char * instrument, t_date date, int antenna)
{
	char *		fname = FNAME ("SDT_open_mmo");
	t_err		error = OK;
	int		status = OK;
	int		i;

	int		sat_no = MMO_ID;

	int		mode = SDT_TLMRATE;
							// 1 : SDT_TLMONLINE
							// 2 : SDT_TLMRATE 
							// 3 : SDT_SIRIUS ???

	int		type = SDT_TLMTIME;
							// 1 : SDT_TLMPASS
							// 2 : SDT_TLMTIME 

	int		cpn_type = 1;			// Transforme en SDT_PACKET !!!


	int		time_kind = TIME_RECEIVED;
							// 1 : TIME_RECEIVED
							// 2 : TIME_GENERATED (Don't seems to work => error -20013)
			
	const	int	ant_band [][2] = {		// List of (antennas, band)
			
		//	ANT_KSC, BAND_X,
		//	ANT_USD, BAND_X,

		//	ANT_53B, BAND_X,
		//	ANT_SOC, BAND_X,

			antenna, BAND_X,
			0, 0
			};

	
	STR_TIME	start_time, stop_time;
	STR_TIME	pass_no;

	blk_no = 16;

	Cpn_data	cpn_buf[16];

	int		cpn_cpt = 0;			// Number of parameters

	t_descr *	descriptor = NULL;

	Message_log (1, fname, "%s %s", instrument, date_to_str (date));

	if (strcmp (instrument, "MEA1") == 0) descriptor = MEA1_DESCR;
	
	if (strcmp (instrument, "MEA2") == 0) descriptor = MEA2_DESCR;

	if (descriptor == NULL) {

		printf ("ERREUR : Instrument incorrect : %s\n", instrument);
		error = ERR_argument;
		goto EXIT;
	}

	SDT_clear();

 	memset ((void *) cpn_buf, 0, sizeof (cpn_buf));

	//	Set VCID_REAL

	for (i = 0; descriptor [i].apid != EOF; i++) {

		// Set VCID

		wd.iwd = htonl (0x3F);

		cpn_buf [i].vc_ch [0] = wd.cwd [3];
	
		wd.iwd = htonl (descriptor [i].vcid);

		cpn_buf [i].vc_ch [1] = wd.cwd [3];
		
		// Set APID

		wd.iwd = htonl (0x7FF);
	
		cpn_buf [i].pc_id [0] = wd.cwd [2];
		cpn_buf [i].pc_id [1] = wd.cwd [3];

		wd.iwd = htonl (descriptor [i].apid);

		cpn_buf [i].pc_id [2] = wd.cwd [2];
		cpn_buf [i].pc_id [3] = wd.cwd [3];
	} 

	cpn_cpt = i;

	sprintf (start_time, "%04d%02d%02d000000", date.year, date.month, date.day);
	sprintf (stop_time,  "%04d%02d%02d235959", date.year, date.month, date.day);
	sprintf (pass_no,    "");

	Message_log (1, fname, "MODE      = %d", mode);
	Message_log (1, fname, "TYPE      = %d", type);
	Message_log (1, fname, "TIME KIND = %d", time_kind);
	Message_log (1, fname, "ANTENNA   = %d", ant_band [0][0]);
	Message_log (1, fname, "BAND      = %d", ant_band [0][1]);
	Message_log (1, fname, "START     = %s", start_time);
	Message_log (1, fname, "STOP      = %s", stop_time);

	status = SDT_tlm_open (mode, 
		type, 
		sat_no, 
		ant_band, 
		cpn_type, 
		cpn_cpt, 
		(uint8_t *) cpn_buf,
		blk_no,
		time_kind,
		start_time,
		stop_time,
		pass_no);

	Message_log (1, fname, "Status = %d", status);

	if (status != 0) { 

		error = ERROR;
		goto EXIT;
	}

	error = SDT_Confirm ();

EXIT:	return error;
}



////////////////////////////////////////////////////////////////////////////////
//
//	Compure mode from Category and Packet ID
//
char *	SC_Mode (int category, int packet_id)
{
	int	pkt_id = packet_id & 0xF;

	switch (category) {

	case 	33:	// H Mode
		if (pkt_id == 0) return "HK";
		break;

	case	51:
		if (pkt_id == 2) return "L";
		break;
	case	61:
		if (pkt_id == 3) return "M";
		break;
	case	71:
	case	81:
	case	91:
	case	101:
	case	111:
	case	121:
		if (pkt_id >= 5 && pkt_id <= 9) return "H";
	}
	return NULL;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Read data packets
//
t_err	SDT_read_packets (char * instrument, t_date date, int antenna)
{
	char *		fname = FNAME ("SDT_read_packets");
	t_err		error = OK;
	int		status = 0;
	FILE *		output = NULL;
	t_filename	output_filename;
	char		key [15];
	int		max_time_out = MAX_TIME_OUT;

	// strcpy (output_filename, Get_L0_filename (instrument, date, "TLM"));

	// create separate output file for each antenna, i
	// that have to be merged to produce an unique input file for L1 process
	//
	sprintf (key, "TLM_%03x", antenna);

	strcpy (output_filename, Get_L0_filename (instrument, date, key, "dat"));

	Message_log (1, fname, "Output = %s", output_filename);

	if ((output = fopen (output_filename, "w")) == NULL) {

		Error_log (fname, "Error creating %s", output_filename);
		error = ERROR;
		goto EXIT;
	}

	while (1) {

		int	i;

		status = READ_sdtp_mmo ();

		if (status == 1) {

			Message_log (1,fname, "Time out %d", max_time_out);

			if (max_time_out > 0) {

				max_time_out --;
				usleep (500 * 1000);
				continue;
			}
			error = WAR_eof;
			goto EXIT;
		}
	
		if (status == 20) {

			Message_log (1, fname, "End of data encountered");
			break;
		}

		if (status != 0) {

			Error_log (fname, "READ_sdtp_mmo = %d", status);
			error = ERROR;

			if (status == 10) {	// AB : 2017/02/04
				error = WARNING;	
				continue;
			}
			break;
		}

		max_time_out = MAX_TIME_OUT;

		Message_log (1, fname, "%2d packets received", ipkt_cnt);

		for (i = 0; i < ipkt_cnt; i++) {

			struct tlm_pkt	* pp = & P_tlm [i];

			int 		apid, len;
			int		categ, pck_id, seq_count;
			int		count;
			int		len_ccsds;

			categ  = pp->ccsds_h2 [4];

			pck_id = pp->ccsds_h2 [5] &0x0F;	// Apparemment seul le demi-octet de poids faible

			apid = (int) (pp->ccsds_h1[0] & 0x07) * 256 + pp->ccsds_h1[1];

			seq_count = (pp->ccsds_h1[2] & 0x3F) * 256 + pp->ccsds_h1[3];

			len_ccsds = pp->ccsds_h1 [4] * 256 + pp->ccsds_h1 [5];

			len = ipkt_len [i];

			Message_log (1, fname, "Pkt %02d : %s Apid : %04X  Seq = %5d Size = %5d Cat = %02d PckID = %1d Len_ccsds = %5d", 
				i+1, 
				BCD_to_date (pp->ad_tim),
				apid, 
				seq_count,
				len, 
				categ, 
				pck_id,
				len_ccsds);

			//	Write date to ouptut file
			//
			count	= fwrite (& len, sizeof (int), 1, output)
				+ fwrite (& P_tlm [i], sizeof (char), len, output);

			//	AB le 2017/12/07 
			//	Forcer ecriture si on laisse tourner en temps reel
			//		
			fflush (output);

			if (count != len + 1) {
				Error_log (fname, "Error writing %d/%d", count, len+1);
				error = ERROR;
				goto EXIT;
			}
		}	
	}

EXIT:	if (output != NULL) fclose (output);
	return error;
}
