/*
 *	SDTP_all.h
 *
 *	SDTP内部使用インクルードファイル
 *
 */

#ifndef	_SDTP_ALL_H_
#define	_SDTP_ALL_H_

#include "SDTP_types.h"
#include "SDTP_pdu.h"

#define	_MSG_SDT( ... )	_MSG( "SDT", __VA_ARGS__ )


/********************************************************
 *	復帰コード
 *******************************************************/
#define	SDT_OK			0		/* 正常復帰 */
#define	SDT_NG			-1		/* 異常復帰 */
#define	SDT_PARAMERR		-20001	/* 入力パラメタエラー */
#define	SDT_SEQERR		-20002	/* 通信シーケンスエラー */
#define	SDT_OPENANSERR		-20003	/* 開設否定応答受信 */
#define	SDT_TIMEOUT		-20004	/* タイムアウト発生 */
/*	#define	SDT_PDULENERR	-20005	ＰＤＵ伝文長エラー */
#define	SDT_RECVSEQERR	-20006	/* シーケンスカウンタエラー */
/*	#define	SDT_FOPENERRSV	-20007	サーバログファイルオープンエラー */
/*	#define	SDT_FOPENERRCL	-20008	クライアントログファイルオープンエラー */
/*	#define	SDT_RCVFREEREQ	2		通信解放依頼ＰＤＵ受信 */
#define	SDT_SYSDOWN		-20009	/* 相手プロセスダウン */
/*	#define	SDT_ENVERR		-20010	環境取得エラー */
#define	SDT_INITERR		-20010	/* SDTnet初期化エラー */


/********************************************************
 *	データＰＤＵ通知情報
 *******************************************************/
#define	SDT_D_BEGIN  0x01	/* 先頭データ */
#define	SDT_D_MID    0x02	/* 中間データ */
#define	SDT_D_END    0x04	/* 最終データ */

/********************************************************
 *	データ通信中詳細状態
 *******************************************************/
#define	SDT_DATAWAIT    0x00	/* 待機中 */
#define	SDT_DATASEND    0x01	/* 送信中 */
#define	SDT_DATARECV    0x02	/* 受信中 */
#define	SDT_DATASDRV    0x03	/* 送受信中 */




/********************************************************
 *	タイマー待ち時間
 *	(環境変数にて設定されていない場合は本値を使用する)
 *******************************************************/
#define	SDT_TIME0		"TIME0"	/*	タイマー０環境変数名	*/
#define	SDT_TIMER0VAL	20		/*	タイマー０待ち時間		*/

/********************************************************
 *	タイマー構造体
 *******************************************************/
typedef struct SDT_TIM {
    int		Tim_sts;	/* 状態 */
    time_t	Tim_val;	/* 待ち時間 */
    time_t	Tim_time;	/* タイムアウト時間 */
} SDT_Tim;

/********************************************************
 *	構造体
 *******************************************************/
typedef struct SDT_INPUT {
    int32_t	    T_mode;		    /*- 転送モード */
    int32_t	    Data_type;	    /*- データ種別 */
    int32_t	    Timeout_ms;	    /*@ タイムアウト時間（ミリ秒） */
    uint8_t	    Moto_id[6];		/*- 送信元識別子 */
    uint8_t	    Saki_id[6];		/*- 送信先識別子 */
	SDT_Tim		Tim[1];			/*@ タイマー構造体 */
} SDT_Input;

/********************************************************
 *	構造体
 *******************************************************/
typedef struct SDT_INSIDE {
    int32_t		status;			/* 通信状態 */
    int32_t		Data_sts;		/* データ通信中状態 */
	int32_t		Free_flag;		/* 通信開放依頼送信状態フラグ */
	int			bClose;
	int			bNotify;
} SDT_Inside;

/********************************************************
 *	構造体
 *******************************************************/
typedef struct SDT_LOWLAY {
    int			Sock_id;	/* ソケット識別子 */
} SDT_Lowlay;

/********************************************************
 *	共通領域構造体
 *******************************************************/
typedef struct SDT_COM{
	SDT_Input	input;
	SDT_Inside	inside;
	SDT_Head	head;
	SDT_Lowlay	lowlay;
} SDT_Com;


int SDTP_open( SDT_Com *com, const uint8_t buff[], int buff_len, int *err_no );
int SDTP_close( SDT_Com *com, const uint8_t buff[], int buff_len, int *err_no );
int SDTP_send( SDT_Com *com, int pdu_type, int rec_num, const uint8_t buff[], int buff_len, int *err_no );
int SDTP_recv( SDT_Com *com, PDU *pdu, int *pdu_len, int *err_no );
int SDTP_reqans( SDT_Com *com, int pdu_type, const uint8_t buff[], int buff_len, PDU *pdu, int *err_no );


#endif	/* _SDTP_ALL_H_ */
