/*
 *	SDTP_pdu.h
 *
 *	SDTP pdu関連インクルードファイル
 *
 */

#ifndef	_SDTP_PDU_H_
#define	_SDTP_PDU_H_

#include <string.h>
#include "SDTP_types.h"

#define	LEN_PDU		(0x10000)
#define	LEN_PDU_H	sizeof(SDT_Head)
#define	LEN_PDU_D	( LEN_PDU - LEN_PDU_H )

/********************************************************
 *	ＳＤＴＰヘッダ構造体
 *******************************************************/
#define	SDT_IDLEN		6		/* 送信元／先識別子長	*/

typedef struct SDT_HEAD {
	uint8_t	Flag_ver;			/* 先頭フラグ／バージョン番号	*/
	uint8_t	Pdu_kind;			/* ＰＤＵ種別					*/
	uint8_t	Send_seq_cnt[4];	/* 送信シーケンスカウンタ		*/
	uint8_t	Recv_seq_cnt[4];	/* 受信シーケンスカウンタ		*/
	uint8_t	Data_kind[2];		/* データ種別					*/
	uint8_t	Moto_id[SDT_IDLEN];	/* 送信元識別子					*/
	uint8_t	Saki_id[SDT_IDLEN];	/* 送信先識別子					*/
	uint8_t	Session_id;			/* セッションＩＤ				*/
	uint8_t	T_mode;				/* 転送モード					*/
	uint8_t	Pdu_len[4];			/* ＰＤＵ長						*/
	uint8_t	Rec_num[2];			/* レコード長					*/
} SDT_Head;

/********************************************************
 *	先頭フラグ／バージョン番号
 *	(Flag_ver)
 *******************************************************/
#define	SDT_FLGVER		0x41	/* 先頭フラグ／バージョン番号 */

/********************************************************
 *	ＰＤＵ種別コード
 *	(Pdu_kind)
 *******************************************************/
#define	SDT_DATA		0x00	/* データＰＤＵ */
#define	SDT_OPENREQ 	0x01	/* 通信開始要求ＰＤＵ */
#define	SDT_OPENANS		0x02	/* 通信開始応答ＰＤＵ */
#define	SDT_CLOSEREQ 	0x03	/* 通信終了要求ＰＤＵ */
#define	SDT_CLOSEANS 	0x04	/* 通信終了応答ＰＤＵ */
#define	SDT_DATAREQ		0x05	/* データ要求ＰＤＵ */
#define	SDT_ECHOREQ		0x06	/* エコー要求ＰＤＵ */
#define	SDT_ECHOANS		0x07	/* エコー応答ＰＤＵ */
#define	SDT_ECHO		0x08	/* エコーＰＤＵ */
#define	SDT_FREEREQ  	0x09	/* 通信開放依頼ＰＤＵ */
#define	SDT_RESET		0x0a	/* リセットＰＤＵ */
#define	SDT_DATAANS		0x10	/* データ応答ＰＤＵ */
#define	SDT_INDINF		0x80	/* 通知情報ＰＤＵ */

#define	SDT_SIRIUS      0x03	/* ＳＩＲＩＵＳインタフェース */

#define	SDTH_set( h, m, v )				memcpy( (h)->m, v, sizeof((h)->m) )
#define	SDTH_get( h, m, v )				memcpy( v, (h)->m, sizeof((h)->m) )

/********************************************************
 *	送信シーケンスカウンタ
 *	(Send_seq_cnt)
 *******************************************************/
#define	SDTH_set_Send_seq_cnt( h, v )	SDTH_set( h, Send_seq_cnt, v )
#define	SDTH_get_Send_seq_cnt( h, v )	SDTH_get( h, Send_seq_cnt, v )

/********************************************************
 *	受信シーケンスカウンタ
 *	(Recv_seq_cnt)
 *******************************************************/
#define	SDTH_set_Recv_seq_cnt( h, v )	SDTH_set( h, Recv_seq_cnt, v )
#define	SDTH_get_Recv_seq_cnt( h, v )	SDTH_get( h, Recv_seq_cnt, v )

/********************************************************
 *	データ種別コード
 *	(Data_kind)
 *******************************************************/
#define	SDTH_set_Data_kind( h, v )		SDTH_set( h, Data_kind, v )
#define	SDTH_get_Data_kind( h, v )		SDTH_get( h, Data_kind, v )

#define	SDT_TLMONLINE 0x01	/* テレメトリオンラインリアルタイム */
#define	SDT_TLMRATE   0x02	/* テレメトリレートバッファリング */
#define	SDT_COMMAND   0x03	/* コマンドデータ */
#define	SDT_STATIONINF 0x05	/* 局運用情報 */
#define	SDT_SEIGYO    0x07	/* 設備制御 */
#define	SDT_KANSHI    0x09	/* 設備監視 */
#define	SDT_RIREKI    0x0b	/* 局運用履歴 */
#define	SDT_UNYOINF    0x0d	/* 局運用通知情報 */
#define	SDT_TUISEKI   0x11	/* 追跡データ（ＫＳＣ２０ｍ） */
#define	SDT_COMMONINF 0x13	/* コマンドモニタ情報 */
#define	SDT_RANGRATE 0x15	/* レンジレートデータ */
#define	SDT_RANG 0x17		/* レンジデータ */
#define	SDT_UNYOU 0x19		/* 局運用（ＫＳＣ２０ｍ） */
#define	SDT_TRACK34S  0x1b	/* SS帯追跡データ（ＫＳＣ３４ｍ） */
#define	SDT_TRACK34X  0x1d	/* SX帯追跡データ（ＫＳＣ３４ｍ） */

/********************************************************
 *	送信元識別子
 *	(Moto_id)
 *******************************************************/
#define	SDTH_set_Moto_id( h, v )		SDTH_set( h, Moto_id, v )
#define	SDTH_get_Moto_id( h, v )		SDTH_get( h, Moto_id, v )

/********************************************************
 *	送信先識別子
 *	(Saki_id)
 *******************************************************/
#define	SDTH_set_Saki_id( h, v )		SDTH_set( h, Saki_id, v )
#define	SDTH_get_Saki_id( h, v )		SDTH_get( h, Saki_id, v )

/********************************************************
 *	転送モード
 *	(T_mode)
 *******************************************************/
#define	SDT_MODE1		0x01	/* 転送モード１ */
#define	SDT_MODE3		0x03	/* 転送モード３ */

/********************************************************
 *	ＰＤＵ長
 *	(Pdu_len)
 *******************************************************/
#define	SDTH_set_Pdu_len( h, v )		SDTH_set( h, Pdu_len, v )
#define	SDTH_get_Pdu_len( h, v )		SDTH_get( h, Pdu_len, v )

/********************************************************
 *	レコード長
 *	(Rec_num)
 *******************************************************/
#define	SDTH_set_Rec_num( h, v )		SDTH_set( h, Rec_num, v )
#define	SDTH_get_Rec_num( h, v )		SDTH_get( h, Rec_num, v )


/*	SDTHバイトオーダ変換 (Net  => Host)	*/
void SDTP_pdu_head_ntoh( SDT_Head *h );
/*	SDTHバイトオーダ変換 (Host => Net )	*/
void SDTP_pdu_head_hton( SDT_Head *h );


/********************************************************
 *	PDU 構造体
 *******************************************************/
typedef struct _PDU {
	SDT_Head	head;
	uint8_t		data[ LEN_PDU_D ];
} PDU;

/*	PDU受信処理	*/
int SDTP_pdu_read( int fd, PDU *pdu, int *pdu_len, int timeout, int *err_no );
/*	PDU送信処理	*/
int SDTP_pdu_write( int fd, PDU *pdu, int pdu_len, int *err_no );


#endif	/* _SDTP_PDU_H_ */
