/********************************************************
 *    各種ファイル名
 *      （環境変数未設定時のシンボルパス名）
 *  変更履歴
 *      2011.02.07  取得先を意識しない・・・（全面改修）
 *******************************************************/
/* ソケット接続状態ファイル名 */ 
#define    T_MODE         SDT_MODE3
#define    SDT_NOTHING    1

/********************************************************
 *    データ取得モード
 *******************************************************/
#define    SDT_TLMPASS    0x01    /* パス番号指定 */
#define    SDT_TLMTIME    0x02    /* 時刻指定 */

/********************************************************
 *    テレメ提供関数用定数
 *******************************************************/
#define    SDT_PASSLEN    10     /* パス番号指定桁数 */
#define    SDT_TIMELEN    14     /* 時刻指定桁数 */

/********************************************************
 *    テレメ関数共通領域構造体
 *******************************************************/
typedef struct SDT_TLM{
    unsigned int    Blk_no;      /* ブロック化係数 */
    unsigned int    Tlm_flg;     /* リアルタイム／レートバッファリング識別 */
    unsigned int    Cpn_flg;     /* 1:パケット 2:ＶＣＤＵ 3:トランスファーフレーム */
} SDT_Tlm;

/********************************************************
 *    ＳＤＴＰユーザ提供関数定義
 *******************************************************/
    /* 共通関数 */
    /* ASCII-CODE TO BCD-CODE */
int  com_ascbcd_(
    char asc[],
    char bcd[],
    int *leng,
    int *ir,
    int *ispos,
    int *iepos
);

     /* ASCII-CODE(HEX) TO BINARY */
int  com_aschex_(
     char asc[],
     char hex[],
     int *leng,
     int *ir,
     int *ispos,
     int *iepos
);

     /* BCD-CODE TO HEX-CODE */
int  com_bcdbin_(
     char bcd[],
     int *bin,
     int *leng,
     int *ipos
);

     /* Devicer */
int  com_devid_(
     int *wme,
     int *wyou,
     int *ans,
     int *amr
);

     /* St date To Total Date */
int  com_ymdtd_(
     char ymday[],
     int *totl_day
);

    /* ＳＤＴＰテレメ開設処理関数 */
int SDT_tlm_open(
    const int   mode,                   /* モード */
    const int   type,                   /* タイプ */
    const int   sat_no,                 /* 衛星番号 */
    const int   ant_band[][2],          /* 受信アンテナ番号と受信バンド帯 */
    const int   cpn_type,               /* ＣＰＮサービス種 */
    const int   cpn_cnt,                /* ＣＰＮサービス個数 */
    const uint8_t *cpn_data,            /* ＣＰＮサービス情報 */
    const int   blk_no,                 /* ブロック化係数 */
    const int   time_kind,              /* 時刻種別 */
    const char *start_time,             /* 開始時刻 */
    const char *end_time,               /* 終了時刻 */
    const char *passno                  /* パス番号 */
);

    /* ＳＤＴＰテレメ解放処理関数 */
int SDT_tlm_close();

    /* ＳＤＴＰテレメ受信処理関数 */
int SDT_tlm_recv(
    int     *pdu_type,             /* 受信ＰＤＵタイプ */
    uint8_t *recv_data,            /* 受信データ */
    int     *recv_len              /* 受信データ長 */
);

/********************************************************
 *    ログファイル出力時のメッセージ
 *******************************************************/
#define    I_TLMOPEN         "Teleme-session-open( OK )"
#define    I_TLMCLOSE        "Teleme-session-close( OK )"
#define    E_TLMOPENNG       "Teleme-session-open( NG )"
#define    E_TLMCLOSENG      "Teleme-session-close( NG )"
#define    E_OPENSOCKPATH    "SOCKPATH-File-Open-Error"
#define    E_ENVERR          "Environment-Set-Error"
#define    E_GHOSTBYNAME     "gethostbyname()-Call-Error"

/********************************************************
 *    データ情報フィールド構造体
 *******************************************************/
typedef struct DATA_INF{
    uint8_t    Rec_kind;      /* レコード識別 */
    uint8_t    Data_id[7];    /* データＩＤ */
    uint8_t    Rec_len[4];    /* レコード長 */
} DATA_Inf;

/********************************************************
 *    データ先頭データデータフィールド構造体
 *******************************************************/
typedef struct DATA_FIRST{
    uint8_t          Path_kind;      /* パス先頭識別 */
    uint8_t          Path_id[5];     /* パス番号 */
    uint8_t          Sat_kind;       /* 衛星番号 */
    uint8_t          Sat_dist[8];    /* 衛星までの距離 */
    unsigned char    Comment[40];    /* コメント */
    uint8_t          Reserve;        /* 予備 */
}DATA_First;

/********************************************************
 *    付加情報部構造体
 *******************************************************/
typedef struct DATA_ADD{
    uint8_t    Ver_id;           /* 装置管理／バージョン番号 */
    uint8_t    Sat_kind;         /* 衛星番号 */
    uint8_t    Recv_time[10];    /* 受信時刻 */
    uint8_t    Recv_band;        /* 受信バンド帯 */
    uint8_t    Status1;          /* 復調装置ステータス１ */
    uint8_t    Status2;          /* 復調装置ステータス２ */
    uint8_t    Lock_sts;         /* ロックステータス */
    uint8_t    Bit_rate;         /* ビットレート */
    uint8_t    Path_id[5];       /* パス番号 */
    uint8_t    Reserve1;         /* 予備 */
    uint8_t    Ant_id;           /* 受信アンテナＩＤ */
    uint8_t    Reserve2[40];     /* 予備 */
}DATA_Add;
   
/********************************************************
 *    データ最終データデータフィールド構造体
 *******************************************************/
typedef struct DATA_END{
    uint8_t    Path_kind;        /* パス終了識別 */
    uint8_t    Path_id[5];       /* パス番号 */
    uint8_t    Sat_kind;         /* 衛星番号 */
    uint8_t    Reserve[13];      /* 予備 */
}DATA_End;
